/***********************************************
     Gauss Program for An Empirical Illustration 
     Reference: Testing for Stochastic Monotonicity by S. LEE, O. LINTON, and Y-J. WHANG 
     Last Edited on 24 September 2008 
***********************************************/

new;

   /*********************************************************
   **  Application: Intergenerational Income Mobility      **
   **********************************************************/


output file = application.out off;

format  /m1 /rd 12,4;

tstart = date;

   /*********************************************************
   ******              Loading Data                **********
   **********************************************************/

/*** Data Loading ***/

   
load data[616,2]=mobility.prn;

   x = data[.,1];       @ log parental income @
   y = ln(data[.,2]);   @ log son's income    @
   

data = sortc(y~x,1);  /* sort based on y */
   y = data[.,1];
   x = data[.,2];
   n = rows(data);


   /*********************************************************
   ******     Parametric Regression Estimates      **********
   **********************************************************/

xreg = ones(n,1)~x;
bhat = invpd(xreg'xreg)*(xreg'y);

bq = {};
tau = 0.1|0.25|0.5|0.75|0.9;

j = 1;
do until j > rows(tau);

tauj = tau[j];

bqr = lad_f(xreg,y,tauj);

bq = bq|bqr[2];

j = j + 1;
endo;

  
output on;
" mean regression estimate " bhat[2];
" quantile regression estimate: 10, 25, 50, 75, 90% ";
bq';
output off;


   /*************************************************************************************
   ***** Bandwidth Selection by the Rule of Thumb (Fan and Gijbels, 1996, p.202)  *******
   *************************************************************************************/

xpoly = ones(n,1)~x~x^2~x^3~x^4;
  h_x = (1.719/0.776)*1.06*stdc(x)*(n^(-1/5));
  w_0 = kde(x,x,h_x);

h_rot_mat = {};

j = 1;
do until j > rows(tau);

tauj = tau[j];

  bqr = lad_f(xpoly,y,tauj);
  res = y - xpoly*bqr;
h_res = (1.719/0.776)*1.06*stdc(res)*(n^(-1/5));

 tmp1 = kde(res,quantile(res,tauj),h_res);
 tmp2 = 2*bqr[3] + 6*bqr[4]*x + 12*bqr[5]*(x^2);

  num = tauj*(1-tauj)*(tmp1^(-2));
  den = sumc((tmp2^2).*w_0);

h_rot = 1.719*((num/den)^(1/5));

h_rot_mat = h_rot_mat|h_rot; 

j = j + 1;
endo;


output on;
" Rule of thumb bandwidths for ";
" local linear quantile regression estimates: 10, 25, 50, 75, 90% ";
h_rot_mat';
output off;


   /*********************************************************
   ******     Nonparametrc Regression Estimates    **********
   **********************************************************/

 xmin = quantile(x,0.01);
 xmax = quantile(x,0.99);

   x0 = seqa(xmin, (xmax-xmin)/19, 20);

yqhat = {};




i = 1;
do until i > rows(x0);

bq = {};

j = 1;
do until j > rows(tau);


bqr = qllre(y,x,x0[i],h_rot_mat[j],tau[j]);

bq = bq~bqr;

j = j + 1;
endo;

yqhat = yqhat|bq;

i = i + 1;
endo;




@ ***** PLOT THE RESULTS ***** @

library pgraph;
graphset;

_ptek="nonpara.tkf";
_pdate = "";

_plegctl = 1;
 _plctrl = 3;
_plegstr = "quantile=0.10\000quantile=0.25\000quantile=0.50\000quantile=0.75\000quantile=0.90";
_pltype = 1|2|3|5|6;
_plwidth = 5;
_pcolor = 1|2|4|5|6;
_pstype= 8|9|10|12|14;

xtics(8,11,0.5,0);
ytics(7,11,0.5,0);


xlabel("  Parental Log Income ");
ylabel("  Son's Log Income ");
title("Local Linear Quantile Regression Estimates");

xy(x0,yqhat);

ret = tkf2ps("nonpara.tkf","nonpara.ps");



   /*********************************************************
   ******     Testing for Stochastic Monotonicity  **********
   **********************************************************/

cons_mat = {1, 0.75, 1.25};
i_cons = 1;
do until i_cons > rows(cons_mat);


/****** Tuning Parameters *************************/


    h = h_rot_mat[3];                /* bandwidth for median local linear regression */
    h = h*cons_mat[i_cons];


 xmin = quantile(x,0.01);
 xmax = quantile(x,0.99);
   zp = seqa(xmin, (xmax-xmin)/19, 20);  



/******* Critical Value ************************/

  lev0 = 0.01;   @ 1% test @
  lev1 = 0.05;   @ 5% test @
  lev2 = 0.10;   @ 10% test @


   ld = 1177/118;

cstar = (8*ld/pi)^(1/2);
 tmp1 = 0.5*ln(cstar*(maxc(zp)-minc(zp))/h);
   bt = tmp1^(1/2) + ln(tmp1)/(8*tmp1^(1/2));
   ap = 4*bt;



x_seq = seqa(2,0.001,2000);
  tmp = exp(-exp(-(x_seq + x_seq^2/(8*bt^2))).*(1+x_seq/(4*bt^2)));

  zv0 = x_seq[minindc(abs(tmp-(1-lev0)))];
  zv1 = x_seq[minindc(abs(tmp-(1-lev1)))];
  zv2 = x_seq[minindc(abs(tmp-(1-lev2)))];
  
  cv0 = bt + (1/ap)*zv0;
  cv1 = bt + (1/ap)*zv1;
  cv2 = bt + (1/ap)*zv2;

" Asymptotic Expansion ";
" 1% critical value  " cv0;
" 5% critical value  " cv1;
" 10% critical value " cv2;


/*-- Alternatively, using the Gumbel distribution --*/

  gcv0 = bt + (1/ap)*ln(1/(-ln(1-lev0)));
  gcv1 = bt + (1/ap)*ln(1/(-ln(1-lev1)));
  gcv2 = bt + (1/ap)*ln(1/(-ln(1-lev2)));



" Asymptotic Distribution ";
" 1% critical value  " gcv0;
" 5% critical value  " gcv1;
" 10% critical value " gcv2;





/******* Supremum Test Statistic ************************/


 yp = y;                                     /* observed Y in [min Y,max Y] */ 



suptest = {};


  i_x = 1;
  do until i_x .> rows(zp);
    
    z = zp[i_x];


 num2 = (kl((x-z)/h)/h)*(kl((x'-z)/h)/h);
 num2 = diagrv(num2, zeros(n,1));              @ exclude diagonal components @


  den = 0;
  
    i = 1;
    do until i > n;
    
      xi = x[i];
    den1 = sgn(xi - x)*sgn(xi - x');
    den1 = diagrv(den1, zeros(n,1));           @ exclude diagonal components @
     
    den1 = ((kl((xi-z)/h)/h)^2)*den1;
    deni = sumc(sumc(den1.*num2));
     den = den + deni;
     
    i = i + 1;
    endo;
    
     den = 4*den/(n*(n-1)*(n-2));     
     den = den + 1e-8*(den <= 1e-8);
     den = sqrt(den/n);




  i_y = 1;
  do until i_y > rows(yp);
      
    t = yp[i_y]; 
    

 num1 = ((y .<= t) - (y' .<= t)).*sgn(x-x');
  num = (sumc(sumc(num1.*num2)))/(n*(n-1));

 test = num/den;

" i_x -- i_y -- test " i_x~i_y~test;

suptest = suptest~test;


i_y = i_y + 1;
 endo;

 
 i_x = i_x + 1;
 endo;
 
 sup_test = maxc(suptest');
 

/******* Printing Results ************************/  



output on;
"===============================================================";
" Supremum  Test ";
" Sample Size -- Bandwidth --  Test Statistic ";
n~h~sup_test;
" Critical Values: Nominal Levels 1% 5% 10% (asymptotic expansion)";
cv0~cv1~cv2;
" Critical Values: Nominal Levels 1% 5% 10% (asymptotic distribution)";
gcv0~gcv1~gcv2;
output off;

i_cons = i_cons + 1;
endo;



  etime = ethsec(tstart,date)/6000;

output on;
" Running time   " etime;
"";
 output off; 
 
end;



   /*********************************************************
   ******                 Procedures               **********
   **********************************************************/


/**********************************************************/
/***************  Sign Function            ****************/
/**********************************************************/

proc sgn(x);   /* Sign function */
local retval;
retval = (x .> 0) - (x .< 0);
retp(retval);
endp;


/**********************************************************/
/*************** 2nd Order Kernel Function ****************/
/**********************************************************/

/**
proc(1) = kl(v);
local term;
term = (15/16)*((1 - v^2)^2);
term = term.*(abs(v) .<= 1);
retp(term);
endp;
**/

proc(1) = kl(x);    
local retval;
retval = (3/4)*(1 - x^2).*(abs(x) .<= 1);
retp(retval);
endp;

/***********  Procedures *********************************/



/*********************************************************/
/******** Kernel Density Estimator         ***************/
/*********************************************************/

proc(1) = kde(x,x_arg,h);  

@
--- Inputs ---
    x = n*d vector of observations
x_arg = k*d vector of arguments of density
    h = d*1 vector of bandwidths
--- Output ---
k*1 vector of kernel density estimates
@

local n,d,k,i,arg,tmp,fhat;

   n = rows(x);
   d = cols(x);
   k = rows(x_arg);

 tmp = ones(k,n);
   i = 1;
   do until i > d;

          arg = (x_arg[.,i]-x[.,i]')/h[i];
          tmp = tmp.*(kl(arg)/h[i]);

   i = i + 1;
   endo;

fhat = meanc(tmp');

retp(fhat);
endp;


/*********************************************************/
/******** Nadaraya-Watson Kernel Estimator ***************/
/*********************************************************/
proc(1) = nwk(y,x,x_arg,h);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*d vector of observations of independent variable
x_arg = k*d vector of arguments of regression function
    h = d*1 vector of bandwidths
--- Output ---
k*1 vector of kernel estimates
@

local n,d,k,i,arg,tmp1,tmp2,m,mhat;

   n = rows(x);
   d = cols(x);
   k = rows(x_arg);

tmp1 = ones(k,n);
   i = 1;
   do until i > d;

          arg = (x_arg[.,i]-x[.,i]')/h[i];
         tmp1 = tmp1.*kl(arg);

   i = i + 1;
   endo;

   m = y';
tmp2 = tmp1.*m;
mhat = sumc(tmp2')./(1.e-10 + sumc(tmp1'));

retp(mhat);
endp;

/**********************************************************/
/**** CV Function for Kernel Mean Regression Estimator ****/
/**********************************************************/
proc(1) = cv_nwk(y,x,h,w);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*d vector of observations of indepedent variable
    h = d*1 vector of bandwidths
    w = weight function (= 1 include; = 0 exclude)
--- Output ---
value of cv function at h
@

local n,i,temp,ei,yi,xi,mh,cvh;

   n = rows(x);

temp = {};

   i = 1;
   do until i > n;

if w[i] .== 1;
 
    ei = zeros(n,1);
 ei[i] = 1;
    yi = delif(y,ei);
    xi = delif(x,ei);

    mh = nwk(yi,xi,x[i,.],h);
  temp = temp|(y[i]-mh)^2;

endif;

   i = i + 1;
   endo;

    cvh = meanc(temp);

retp(cvh);
endp;

   /*********************************************************/
   /*    Local Linear Mean Regression Estimator             */
   /*********************************************************/

proc(1) = mllre(y,x,x0,h);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*d vector of observations of independent variables
   x0 = d*1 vector where a regression function is evaluated
    h = d*1 vector of bandwidths
--- Output ---
a local linear regression estimate evaluated at x0
@

local n,d,w,z,i,arg,b,mhat;

  n = rows(x);
  d = cols(x);

  w = ones(n,1);
  z = ones(n,1);

   i = 1;
   do until i > d;

          arg = (x[.,i]-x0[i]);
            w = w.*(kl(arg/h[i])/h[i]);
            z = z~arg;

   i = i + 1;
   endo;

   w = w + 1e-5*(w.==0);  @ This is to avoid singularity of z'w*z. @
   w = diagrv(eye(n),w);
   b = invpd(z'w*z)*(z'w*y);

mhat = b[1];

retp(mhat);
endp;


   /*********************************************************/
   /*    Local Linear Quantile Regression Estimator         */
   /*********************************************************/

proc(1) = qllre(y,x,x0,h,tau);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*d vector of observations of independent variables
   x0 = d*1 vector where a regression function is evaluated
    h = d*1 vector of bandwidths
  tau = the quantile of interest 
--- Output ---
a local linear quantile regression estimate evaluated at x0
@

local n,d,w,z,i,arg,b,mhat;

  n = rows(x);
  d = cols(x);

  w = ones(n,1);
  z = ones(n,1);

   i = 1;
   do until i > d;

          arg = (x[.,i]-x0[i]);
            w = w.*(kl(arg/h[i])/h[i]);
            z = z~arg;

   i = i + 1;
   endo;

   w = w + 1e-5*(w.==0);       @ This is to avoid singularity of z'w*z. @ 
   w = diagrv(eye(n),w);
   b = lad_f(w*z, w*y, tau);  

mhat = b[1];

retp(mhat);
endp;


   /*********************************************************/
   /*    Linear Quantile Regression Estimator               */
   /*********************************************************/

proc (1) = lad_f(_x, y, q);

local limit, _n, _k, labelout, labelin, c, ind, _t, criterion, margcost, s,
     count, loc, r, init, move, blabel, b;

limit = 100;

_n = rows(_x); _k = cols(_x);

labelout = seqa(1, 1, _k)';

labelin = seqa(_k+1, 1, _n)';

c = q * ones(1, _n);


ind = indexcat(y .< 0,1);

if rows(ind) > 0;

  y[ind,1] = - y[ind,1];
  _x[ind,.] = - _x[ind,.];
  labelin[1,ind] = -labelin[1,ind];
  c[1,ind] = - c[1,ind] + 1.0;

 endif;


_t = (_x ~ y) | (c*_x ~ c * y);

clear _x; clear y; clear c;

criterion = _t[_n+1,1:_k] ~ - _t[_n+1,1:_k];
margcost = maxc(criterion');
s = indexcat(criterion', margcost);
count = 0;
for i (1, _k, 1);

  if s > _k;
  s = s - _k; _t[1:_n,s] = - _t[1:_n,s];
 _t[_n+1, s] = margcost;
  labelout[1,s] = - labelout[1,s];

  endif;

 ind = indexcat( (_t[1:_n, s] .> 0 ) .* (abs(labelin') .> _k), 1);
 if rows(ind) > 0;

  loc = sortind(_t[ind,_k+1]./_t[ind, s]);
  r = ind[loc[1,1], 1];
  init = 1;
  do while ((margcost - _t[r,s] > 0) and (init < rows(ind)));

    _t[_n+1,.] = _t[_n+1,.] - _t[r,.];
    margcost = _t[_n+1,s];
    _t[r,.] = - _t[r,.];
    labelin[1,r] = - labelin[1,r];
    init = init + 1;
    r = ind[loc[init,1],1];
    endo;
    if cols(_t) == _k+2;
   _t[.,_k+2] = (seqa(1,1,_n+1) .== r);
   else;
    _t = _t~(seqa(1,1,_n+1) .== r);
  endif;
  
  _t[r,.] = _t[r,.]/_t[r,s];
  ind = indexcat((seqa(1,1,_n+1) ./= r),1);
  _t[ind,.] = _t[ind,.] - (_t[ind,s]/ _t[r,s]) * _t[r,.];
  _t[.,s] = _t[.,_k+2];

  move = labelout[1,s];
  labelout[1,s] = labelin[1,r];
  labelin[1,r] = move;
  endif;

  if i < _k;
   ind = indexcat(abs(labelout') .<= _k, 1)';
   criterion = _t[_n+1, ind] ~ - _t[_n+1, ind];
   margcost = maxc(criterion'); 
   loc = indexcat(criterion', margcost); 
   if loc <= cols(ind);
   s = ind[1,loc];
   else;
   s = _k + ind[1,loc-cols(ind)]; 
   endif;
   count = count + 1;
   else;

    criterion = _t[_n+1, 1:_k] ~ (- _t[_n+1, 1:_k] - 1);
    margcost = maxc(criterion');
    s = indexcat(criterion', margcost);
    count = count + 1;
  endif;

  endfor;

do while ((margcost > 10 * 2.2204e-016) and (count <= limit));
   if s > _k;
    s = s- _k; _t[1:_n, s] = - _t[1:_n, s];
    _t[_n+1,s] = margcost;                          
    labelout[1,s] = - labelout[1,s];
  endif;

  ind = indexcat( (_t[1:_n, s] .> 0) .* (abs(labelin') .> _k), 1);  
  loc = sortind(_t[ind, _k+1] ./ _t[ind, s] );
  r = ind[loc[1,1],1];
  init = 1;

  do while ((margcost - _t[r,s] > 0) and (init < rows(ind) ));
    _t[_n+1,.] = _t[_n+1,.] - _t[r,.];
    margcost = _t[_n+1,s];
   _t[r,.] = - _t[r,.];
   labelin[1,r]= - labelin[1,r];
   init = init + 1;
   r = ind[loc[init,1],1];
  endo;

  if cols(_t) == _k+2;
   _t[.,_k+2] = (seqa(1,1,_n+1) .== r);
   else;
  _t = _t~(seqa(1,1,_n+1) .== r);
  endif;

  _t[r,.] = _t[r,.]/_t[r,s];
  ind = indexcat((seqa(1,1,_n+1) ./= r), 1)';
  _t[ind,.] = _t[ind,.] - (_t[ind,s]/ _t[r,s]) * _t[r,.];
  _t[.,s] = _t[.,_k+2];
  move = labelout[1,s];
  labelout[1,s] = labelin[1,r];
  labelin[1,r] = move;
  criterion = _t[_n+1, 1:_k] ~ (- _t[_n+1, 1:_k] - 1);
  margcost = maxc(criterion');
  s = indexcat(criterion', margcost);
  count = count + 1;
  endo;

  ind = indexcat(abs(labelin') .<= _k, 1)';
  blabel = labelin[1,ind]';
  b = _t[ind, _k+1] .* ((blabel .> 0) - (blabel .< 0));
  ind = sortind(abs(blabel));
  b = b[ind,1];
  retp(b);

  endp;
